package dev.fitko.fitconnect.cli;

import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.client.SenderClient;
import dev.fitko.fitconnect.client.bootstrap.ApplicationConfigLoader;
import dev.fitko.fitconnect.client.bootstrap.ClientFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.file.Path;
import java.util.stream.Collectors;

public final class CommandLineRunner {

    private static final String LOGO = "/splash_screen_banner.txt";
    private static final String DEFAULT_CONFIG_NAME = "config.yml";

    private static final Logger LOGGER = LoggerFactory.getLogger(CommandLineRunner.class);

    private CommandLineRunner() {
    }

    public static void main(final String[] args) {
        printSplashScreen();
        getCommandLineClient(loadConfig()).run(args);
    }

    private static ApplicationConfig loadConfig() {
        try {
            return ApplicationConfigLoader.loadConfigFromEnvironment();
        } catch (final Exception e) {
            LOGGER.warn("Could not load config from environment, loading default config {}", DEFAULT_CONFIG_NAME);
            return ApplicationConfigLoader.loadConfigFromPath(Path.of(DEFAULT_CONFIG_NAME));
        }
    }

    private static CommandLineClient getCommandLineClient(final ApplicationConfig config) {
        final SenderClient senderClient = ClientFactory.getSenderClient(config);
        final var subscriberClient = ClientFactory.getSubscriberClient(config);
        return new CommandLineClient(senderClient, subscriberClient);
    }

    private static void printSplashScreen() {
        LOGGER.info("{}", getSplashScreenResource());
    }

    private static String getSplashScreenResource() {
        final InputStream is = CommandLineRunner.class.getResourceAsStream(LOGO);
        return new BufferedReader(new InputStreamReader(is)).lines().collect(Collectors.joining("\n"));
    }
}
